#include <netlink/msg.h>
#include "netlink/netlink.h"

#include "common/types.h"
#include "logger/logger.h"

#include "callback/data.h"
#include "callback/callback.h"

static int (* localHandler)(struct nl_msg* message, void* arg) = NULL;
static struct callbackData* localData = NULL;
static int localError = 0;

struct nl_cb* netlinkCallback = NULL;

extern struct nl_sock *socketInstance;

void registerHandler(int (* handler)(struct nl_msg*, struct callbackData* data),
					 struct callbackData* data) {
	localHandler = (int (*)(struct nl_msg* message, void* arg))handler;
	localData = data;
}

static int handle(struct nl_msg* message, void* _) {
	if(localHandler == NULL) {
		LOG(WARN, "no handler specified");
		return NL_OK;
	} else {
		return localHandler(message, (void*)localData);
	}
}

static int handleError(struct sockaddr_nl *nla,
						   struct nlmsgerr *nlerr, void *arg) {
	fprintf(stderr, "ERROR %s\n", nl_geterror(nlerr->error));
	return NL_STOP;
}

static int handleFinish(struct nl_msg *_, void *arg) {
	fprintf(stderr, "FINISH\n");
	*((int*)arg) = 0;
	return NL_SKIP;
}

static int handleAck(struct nl_msg *msg, void *arg) {
	struct nlmsghdr *header = nlmsg_hdr(msg);
	fprintf(stderr, "ACK, flags=%d, type=%d\n", header->nlmsg_flags, header->nlmsg_type);
	*((int*)arg) = 0;
	return NL_STOP;
}

void initCallback(bool isDebug) {
	if (isDebug) {
		netlinkCallback = nl_cb_alloc(NL_CB_DEBUG);
	} else {
		netlinkCallback = nl_cb_alloc(NL_CB_DEFAULT);
	}

	nl_cb_err(netlinkCallback, NL_CB_CUSTOM, handleError, &localError);
	nl_cb_set(netlinkCallback, NL_CB_FINISH, NL_CB_CUSTOM, handleFinish, &localError);
	nl_cb_set(netlinkCallback, NL_CB_ACK, NL_CB_CUSTOM, handleAck, &localError);
	nl_cb_set(netlinkCallback, NL_CB_VALID, NL_CB_CUSTOM, handle, NULL);
}
