#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common/types.h"

#include "label.h"

static const char* boolName[2] = {
	"false",
	"true",
};

static LoggerLabel* labelInit(const char* name, LoggerLabelType type) {
	LoggerLabel* label = calloc(1, sizeof(LoggerLabel));

	unsigned long nameLength = strlen(name);

	label->type = type;
	label->name = calloc(nameLength + 1, sizeof(char));
	strncpy(label->name, name, nameLength);

	return label;
}

LoggerLabel* LabelError(const char* value) {
	return LabelString("err", value);
}

LoggerLabel* LabelString(const char* name, const char* value) {
	LoggerLabel* label = labelInit(name, labelString);

	unsigned long valueLength = strlen(value);
	label->value.s = calloc(valueLength + 1, sizeof(char));
	strncpy(label->value.s, value, valueLength);

	return label;
}

LoggerLabel* LabelInt(const char* name, int value) {
	LoggerLabel* label = labelInit(name, labelInt);

	label->value.i = value;

	return label;
}

LoggerLabel* LabelFloat(const char* name, float value) {
	LoggerLabel* label = labelInit(name, labelFloat);

	label->value.f = value;

	return label;
}

LoggerLabel* LabelDouble(const char* name, double value) {
	LoggerLabel* label = labelInit(name, labelDouble);

	label->value.d = value;

	return label;
}

LoggerLabel* LabelBool(const char* name, bool value) {
	LoggerLabel* label = labelInit(name, labelBool);

	label->value.b = value;

	return label;
}

void LabelFree(LoggerLabel* label) {
	switch(label->type) {
		case labelString:
			free(label->value.s);
			break;
		default:
			break;
	}

	free(label);
}

void LabelPrint(LoggerLabel* label) {
	printf("%s=", label->name);
	switch(label->type) {
		case labelString:
			printf("%s", label->value.s);
			break;
		case labelInt:
			printf("%d", label->value.i);
			break;
		case labelFloat:
			printf("%f", label->value.f);
			break;
		case labelDouble:
			printf("%f", label->value.d);
			break;
		case labelBool:
			printf("%s", boolName[label->value.b]);

		default:
			break;
	}
}
