#include <errno.h>
#include <string.h>

#include <net/if.h>
#include <linux/nl80211.h>

#include "netlink/genl/genl.h"

#include "callback/callback.h"

#include "common/indication.h"

#include "message/get/interface.h"
#include "message/receive.h"
#include "message/message.h"

extern int driverId;
extern struct nl_sock *socketInstance;

int getInterfaceModeCallback(struct nl_msg *message, struct callbackData *result) {
	struct nlmsghdr *header = nlmsg_hdr(message);
	struct nlattr *attributeList[NL80211_ATTR_MAX + 1];

	if (header->nlmsg_type != driverId) {
		return NL_STOP;
	}

	struct genlmsghdr *payloadLocation = (struct genlmsghdr*)nlmsg_data(header);

	nla_parse(attributeList, NL80211_ATTR_MAX, genlmsg_attrdata(payloadLocation, 0), genlmsg_attrlen(payloadLocation, 0), NULL);

	if (attributeList[NL80211_ATTR_IFTYPE] != NULL) {
		enum nl80211_iftype type = nla_get_u32(attributeList[NL80211_ATTR_IFTYPE]);
		*((enum nl80211_iftype*)result->data) = type;
		printf("==============+>%d\n", type);
	}

	return NL_OK;
}

__ERROR_STRICTLY_NEGATIVE enum nl80211_iftype getInterfaceMode(const char *interfaceName) {
	struct nl_msg *message = createInterfaceMessage(interfaceName,
													NL80211_CMD_GET_INTERFACE);

	enum nl80211_iftype type = -1;
	struct callbackData resultData = {
		.result = 0,
		.data = &type,
	};

	registerHandler(getInterfaceModeCallback, &resultData);

	int result;
	if ((result = sendMessage(message))) {
		return result;
	}

	receive();

	return type;
}