#include <string.h>
#include <errno.h>

#include <linux/nl80211.h>

#include <net/if.h>

#include <netlink/msg.h>
#include <netlink/genl/genl.h>

#include "common/indication.h"
#include "common/types.h"

#include "logger/logger.h"

#include "message.h"

extern struct nl_sock *socketInstance;
extern int driverId;

struct nl_msg *createInterfaceMessage(const char *interfaceName,
									  enum nl80211_commands command) {
	unsigned int interfaceIndex = if_nametoindex(interfaceName);
	if (interfaceIndex == 0) {
		LOG(ERROR, "interface not found",
			LabelError(strerror(errno)),
			LabelString("interfaceName", interfaceName));
		return NULL;
	}

	struct nl_msg *message = nlmsg_alloc();

	genlmsg_put(message, NL_AUTO_PORT, NL_AUTO_SEQ, driverId, 0, 0, command, 0);

	int error;
	if ((error = nla_put_u32(message, NL80211_ATTR_IFINDEX, interfaceIndex)) != 0) {
		LOG(ERROR, "could not put interface index in message",
			LabelError(nl_geterror(error)),
			LabelString("interfaceName", interfaceName));
		return NULL;
	}

	return message;
}

nl_result sendMessage(__WILL_BE_OWNED struct nl_msg *message) {
	int error;
	if ((error = nl_send_auto(socketInstance, message)) <= 0) {
		LOG(ERROR, "could not send message",
			LabelError(nl_geterror(error)));
		return error;
	}
	nlmsg_free(message);
	return 0;
}
